package org.jboss.soa.qa.jpadroolsrollback;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.importer.ZipImporter;
import org.jboss.shrinkwrap.api.spec.JavaArchive;
import org.jboss.soa.qa.jpadroolsrollback.domain.InEntity;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.transaction.UserTransaction;
import java.io.File;
import java.util.zip.ZipFile;

@RunWith(Arquillian.class)
public class JPADroolsRollbackTest {

	private static final String JAR_FILE = "target/jpa-drools-rollback-issue.jar";

	@Deployment
	public static Archive<?> createTestArchive() throws Exception {
		File artifact = new File(JAR_FILE);
		try {
			return ShrinkWrap.create(ZipImporter.class, artifact.getName()).importFrom(new ZipFile(artifact)).as(JavaArchive.class);
		} catch (Exception e) {
			throw new RuntimeException(JAR_FILE + " not found. Run 'mvn package'", e);
		}
	}

	@PersistenceContext
	private EntityManager em;

	@Inject
	UserTransaction utx;

	@Test
	public void testJPABindingTransaction() throws Exception {
		utx.begin();
		InEntity entity = new InEntity();
		entity.setCommand("test");
		em.persist(entity);
		utx.commit();

		Thread.sleep(5000); // wait for rollbacks and final commit

		// DBEntity processed
		Assert.assertEquals(0, em.createQuery("SELECT e FROM InEntity e").getResultList().size());
	}
}
